import { Container } from '@microsoft/fast-foundation';
import { NodeEnv, NodeEnvSchema } from './nodeEnv';
import { Percentage, PercentageSchema } from './percentage';
import { TimeWindow, TimeWindowSchema } from './timeWindow';
import { URLTargeting, URLTargetingSchema } from './urlTargeting';
import { UserAgent, UserAgentSchema } from './userAgent';
import { UserTargeting, UserTargetingSchema } from './userTargeting';
/**
 * @public
 */
export declare const ClientFilterName: {
    readonly nodeEnv: "nodeEnv";
    readonly percentage: "percentage";
    readonly timeWindow: "timeWindow";
    readonly urlTargeting: "urlTargeting";
    readonly userAgent: "userAgent";
    readonly userTargeting: "userTargeting";
};
/**
 * @public
 */
export type ClientFilterName = (typeof ClientFilterName)[keyof typeof ClientFilterName];
/**
 * Filer lookup by name.
 *
 * @public
 */
export declare function getFilterByName(name: ClientFilterName, container: Container): NodeEnv | Percentage | TimeWindow | URLTargeting | UserAgent | UserTargeting;
/**
 * ClientFilter interface.
 *
 * @remarks
 * Simplifies access to runtime data / state contained within the DI container.
 *
 * @typeParam P - The filter parameter type.
 *
 * @public
 */
export interface ClientFilter<P = any> {
    /**
     * @param parameters - The filter parameters.
     *
     * @returns The filter outcome.
     *
     * @privateRemarks
     * Not sure if there's a need for an async api here.
     */
    filter(parameters: P): boolean;
}
/**
 * ClientFilterSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @typeParam N - The filter name type.
 * @typeParam P - The filter parameters type.
 *
 * @public
 */
export type ClientFilterSchema<N extends ClientFilterName, P> = {
    readonly name: N;
    readonly parameters: P;
};
/**
 * ClientFilterSchemas type.
 *
 * @public
 */
export type ClientFilterSchemas = NodeEnvSchema | PercentageSchema | TimeWindowSchema | URLTargetingSchema | UserAgentSchema | UserTargetingSchema;
/**
 * Client filter runner function.
 * @remarks
 * This will return true when all the filters in the array pass.
 * @param filters - An array of client filters to run.
 * @param container - The DI container.
 * @public
 * @privateRemarks
 * As a runner working off static input, we can't know or source the runtime dependencies of each filter. As such we use
 * the DI version of each which have those encapsulated.
 */
export declare function runner(filters: ClientFilterSchemas[], container: Container): boolean;
/**
 * ClientFilterRunner DI interface.
 *
 * @public
 */
export interface ClientFilterRunner {
    run(filters: ClientFilterSchemas[], container?: Container): boolean;
}
/**
 * Default ClientFilterRunner DI implementation.
 * @public
 */
export declare class DefaultClientFilterRunner implements ClientFilterRunner {
    container: Container;
    run(filters: ClientFilterSchemas[], container?: Container): boolean;
}
/**
 * ClientFilterRunner DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const ClientFilterRunner: import("@microsoft/fast-foundation").InterfaceSymbol<ClientFilterRunner>;
//# sourceMappingURL=clientFilter.d.ts.map