import type { ClientFilter, ClientFilterName, ClientFilterSchema } from './clientFilter';
/**
 * Percentage client filter function.
 * @remarks
 * This will return true a certain percentage of the time.
 * @param percent - The percent of the time to pass, a numeric value between 0 and 1.
 * @param random - Math.random() generated floating-point.
 * @public
 */
export declare function percentageFilter(percent: number, random?: number): boolean;
/**
 * PercentageParams
 *
 * @remarks
 * Percent should be a numeric value between 0 and 1.
 *
 * @example
 * ```json
 * {
 *    "percent": 0.5
 * }
 * ```
 *
 * @example
 * In a rare case where you need to control the random floating-point creation, you can do so externally and provide it.
 * ```json
 * {
 *    "percent": 0.34
 *    "random": 0.08901819974272063
 * }
 * ```
 *
 * @public
 */
export type PercentageParams = {
    readonly percent: number;
    /**
     * @remarks
     * Provided for fine-grain control over the random floating-point
     */
    readonly random?: number;
};
/**
 * PercentageSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @public
 */
export type PercentageSchema = ClientFilterSchema<typeof ClientFilterName.percentage, PercentageParams>;
/**
 * Percentage client filter DI interface.
 *
 * @remarks
 * Simplifies access to runtime data / state contained within the DI container.
 *
 * @public
 * @privateRemarks
 * Created for consistency with other filters that required DI container access.
 */
export interface Percentage extends ClientFilter<PercentageParams> {
}
/**
 * Default Percentage DI implementation.
 * @public
 */
export declare class DefaultPercentage implements Percentage {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params: PercentageParams): boolean;
}
/**
 * Percentage DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const Percentage: import("@microsoft/fast-foundation").InterfaceSymbol<Percentage>;
//# sourceMappingURL=percentage.d.ts.map