import type { ClientFilter, ClientFilterName, ClientFilterSchema } from './clientFilter';
/**
 * TimeWindow client filter function.
 * @remarks
 * This will return true between a certain time window. Start and end can be provided in any format recognized by the
 * Date.parse() method.
 *
 * @param start - The start of the time window.
 * @param end - The end of the time window.
 * @public
 */
export declare function timeWindowFilter(start?: string, end?: string): boolean;
/**
 * TimeWindowParams
 *
 * @remarks
 * Both params can be provided in any format recognized by the Date.parse() method.
 *
 * @example
 * ```json
 * {
 *   "start": "Sun Sep 25 2022 16:51:55 GMT+0000",
 *   "end": "Fri Nov 25 2022 17:51:55 GMT+0000"
 * }
 * ```
 *
 * @example
 * ```json
 * {
 *   "start": "2022-10-01",
 *   "end": "2022-12-01"
 * }
 * ```
 *
 * @example
 * ```json
 * {
 *   "end": "Dec 31, 2022"
 * }
 * ```
 *
 * @example
 * ```json
 * {
 *   "start": "2023-01-01T00:00:00.000+00:00"
 * }
 * ```
 *
 * @public
 */
export type TimeWindowParams = {
    readonly start?: string;
    readonly end?: string;
};
/**
 * TimeWindowSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @public
 */
export type TimeWindowSchema = ClientFilterSchema<typeof ClientFilterName.timeWindow, TimeWindowParams>;
/**
 * TimeWindow client filter DI interface.
 *
 * @remarks
 * Simplifies access to runtime data / state contained within the DI container.
 *
 * @public
 */
export interface TimeWindow extends ClientFilter<TimeWindowParams> {
}
/**
 * Default TimeWindow DI implementation.
 * @public
 */
export declare class DefaultTimeWindow implements TimeWindow {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params: TimeWindowParams): boolean;
}
/**
 * TimeWindow DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const TimeWindow: import("@microsoft/fast-foundation").InterfaceSymbol<TimeWindow>;
//# sourceMappingURL=timeWindow.d.ts.map