import type { ClientFilter, ClientFilterName, ClientFilterSchema } from './clientFilter';
/**
 * URLTargeting client filter function.
 * @remarks
 * This will return true when the current URL matches on ANY of the parameters. If you wish to match on multiple
 * parameters, consider chaining an array of urlTargeting filters together.
 * @param url - A url.
 * @param pattern - A regex.
 * @param schemes - Schemes/protocols to target.
 * @param hosts - Hosts to target.
 * @param paths - Paths to target.
 * @public
 * @privateRemarks
 * The pattern regex can cover everything here, but some devs will likely welcome the explicit arguments.
 */
export declare function urlTargetingFilter(url: string, pattern?: RegExp, schemes?: string[], hosts?: string[], paths?: string[]): boolean;
/**
 * URLTargetingParams
 *
 * @example
 * ```json
 * {
 *    "schemes": ["https"],
 *    "hosts": ["neptx.genesislab.global", "neptune-uat2-axes-server2"]
 * }
 * ```
 *
 * @public
 */
export type URLTargetingParams = {
    /**
     * A url.
     */
    readonly url?: string;
    /**
     * A regex.
     */
    readonly pattern?: RegExp;
    /**
     * Schemes/protocols to target.
     */
    readonly schemes?: string[];
    /**
     * Hosts to target.
     */
    readonly hosts?: string[];
    /**
     * Paths to target.
     */
    readonly paths?: string[];
};
/**
 * URLTargetingSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @public
 */
export type URLTargetingSchema = ClientFilterSchema<typeof ClientFilterName.urlTargeting, URLTargetingParams>;
/**
 * URLTargeting client filter DI interface.
 *
 * @public
 */
export interface URLTargeting extends ClientFilter<URLTargetingParams> {
    /**
     * @remarks
     * Check if a url is secure.
     *
     * @param url - A url.
     */
    isSecure(url: string): boolean;
}
/**
 * Default URLTargeting DI implementation.
 * @public
 */
export declare class DefaultURLTargeting implements URLTargeting {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params: URLTargetingParams): boolean;
    /** {@inheritDoc URLTargeting.isSecure} */
    isSecure(url: any): boolean;
}
/**
 * URLTargeting DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const URLTargeting: import("@microsoft/fast-foundation").InterfaceSymbol<URLTargeting>;
//# sourceMappingURL=urlTargeting.d.ts.map