import type { ClientFilter, ClientFilterName, ClientFilterSchema } from './clientFilter';
/**
 * UserAgent client filter function.
 *
 * @remarks
 * See {@link https://github.com/faisalman/ua-parser-js} for more information.
 *
 * @param browsers - Browser names to target.
 * @param devices - Device types to target.
 * @param oss - Operating system names to target.
 * @param ua - User-agent string (required in nodejs environment).
 * @public
 */
export declare function userAgentFilter(browsers?: string[], devices?: string[], oss?: string[], ua?: string): boolean;
/**
 * UserAgentParams
 *
 * @example
 * ```json
 * {
 *    "browsers": ["Safari", "Firefox", "Chrome", "Chromium"],
 *    "devices": ["mobile", "tablet"],
 *    "oss": ["Mac OS", "Linux"]
 * }
 * ```
 *
 * @public
 */
export type UserAgentParams = {
    /**
     * Browser names to target.
     */
    readonly browsers?: string[];
    /**
     * Device types to target.
     */
    readonly devices?: string[];
    /**
     * Operating system names to target.
     */
    readonly oss?: string[];
    /**
     * @remarks
     * Provided for fine-grain control over the user-agent string in browser, and nodejs usage.
     */
    readonly ua?: string;
};
/**
 * UserAgentSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @public
 */
export type UserAgentSchema = ClientFilterSchema<typeof ClientFilterName.userAgent, UserAgentParams>;
/**
 * UserAgent client filter DI interface.
 *
 * @remarks
 * Simplifies access to runtime data / state contained within the DI container.
 *
 * @public
 * @privateRemarks
 * Created for consistency with other filters that required DI container access.
 */
export interface UserAgent extends ClientFilter<UserAgentParams> {
    /**
     * @remarks
     * Is the user-agent safari.
     *
     * @privateRemarks
     * Added as an example, see comment in implementation.
     */
    readonly isSafari: boolean;
}
/**
 * Default UserAgent DI implementation.
 * @public
 */
export declare class DefaultUserAgent implements UserAgent {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params: UserAgentParams): boolean;
    /** {@inheritDoc UserAgent.isSafari} */
    get isSafari(): boolean;
}
/**
 * UserAgent DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const UserAgent: import("@microsoft/fast-foundation").InterfaceSymbol<UserAgent>;
//# sourceMappingURL=userAgent.d.ts.map