import { Auth, User } from '@genesislcap/foundation-comms';
import type { ClientFilter, ClientFilterName, ClientFilterSchema } from './clientFilter';
/**
 * UserTargeting client filter function.
 * @remarks
 * This will return true when the current user matches on ANY of the parameters. If you wish to match on multiple
 * parameters, consider chaining an array of userTargeting filters together.
 * @param user - A user object.
 * @param usernames - Usernames to target.
 * @param profiles - Profiles to target.
 * @param permissions - Permissions to target.
 * @public
 * @privateRemarks
 * We could provide another parameter to indicate a complete match is required?
 */
export declare function userTargetingFilter(user: User, usernames?: string[], profiles?: string[], permissions?: string[]): boolean;
/**
 * UserTargetingParams
 *
 * @example
 * ```json
 * {
 *    "usernames": ["JohnDoe"],
 *    "profiles": ["ADMIN", "READ_WRITE_DEALER"],
 *    "permissions": ["ExportClientDashboard"]
 * }
 * ```
 *
 * @public
 */
export type UserTargetingParams = {
    /**
     * Usernames to target.
     */
    readonly usernames?: string[];
    /**
     * Profiles to target.
     *
     * @remarks
     * Profiles are assigned to users, and users may have multiple profiles assigned to them.
     */
    readonly profiles?: string[];
    /**
     * Permissions to target.
     *
     * @remarks
     * Permissions are assigned to profiles, NEVER to users directly.
     * The user permissions array will contain everything from the assigned profiles for fine-grained control.
     */
    readonly permissions?: string[];
};
/**
 * UserTargetingSchema type.
 *
 * @remarks
 * JSON schema type.
 *
 * @public
 */
export type UserTargetingSchema = ClientFilterSchema<typeof ClientFilterName.userTargeting, UserTargetingParams>;
/**
 * UserTargeting client filter DI interface.
 *
 * @remarks
 * Simplifies access to runtime data / state contained within the DI container.
 *
 * @public
 */
export interface UserTargeting extends ClientFilter<UserTargetingParams> {
    /**
     * @remarks
     * Has the user got the 'ADMIN' profile assigned.
     *
     * @privateRemarks
     * Added as an example, see comment in implementation.
     */
    readonly hasAdminProfile: boolean;
    /**
     * @remarks
     * Has the user logged in, regardless of their identity.
     */
    readonly isLoggedIn: boolean;
}
/**
 * Default UserTargeting DI implementation.
 * @public
 */
export declare class DefaultUserTargeting implements UserTargeting {
    auth: Auth;
    constructor(auth: Auth);
    /** {@inheritDoc ClientFilter.filter} */
    filter(params: UserTargetingParams): boolean;
    /** {@inheritDoc UserTargeting.hasAdminProfile} */
    get hasAdminProfile(): boolean;
    /** {@inheritDoc UserTargeting.isLoggedIn} */
    get isLoggedIn(): boolean;
}
/**
 * UserTargeting DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const UserTargeting: import("@microsoft/fast-foundation").InterfaceSymbol<UserTargeting>;
//# sourceMappingURL=userTargeting.d.ts.map