import { __decorate } from "tslib";
import { Container, DI } from '@microsoft/fast-foundation';
import { logger } from './logger';
import { NodeEnv } from './nodeEnv';
import { Percentage } from './percentage';
import { TimeWindow } from './timeWindow';
import { URLTargeting } from './urlTargeting';
import { UserAgent } from './userAgent';
import { UserTargeting } from './userTargeting';
/**
 * @public
 */
export const ClientFilterName = {
    nodeEnv: 'nodeEnv',
    percentage: 'percentage',
    timeWindow: 'timeWindow',
    urlTargeting: 'urlTargeting',
    userAgent: 'userAgent',
    userTargeting: 'userTargeting',
};
/**
 * Filer lookup by name.
 *
 * @public
 */
export function getFilterByName(name, container) {
    switch (name) {
        case ClientFilterName.nodeEnv:
            return container.get(NodeEnv);
        case ClientFilterName.percentage:
            return container.get(Percentage);
        case ClientFilterName.timeWindow:
            return container.get(TimeWindow);
        case ClientFilterName.urlTargeting:
            return container.get(URLTargeting);
        case ClientFilterName.userAgent:
            return container.get(UserAgent);
        case ClientFilterName.userTargeting:
            return container.get(UserTargeting);
    }
}
/**
 * Client filter runner function.
 * @remarks
 * This will return true when all the filters in the array pass.
 * @param filters - An array of client filters to run.
 * @param container - The DI container.
 * @public
 * @privateRemarks
 * As a runner working off static input, we can't know or source the runtime dependencies of each filter. As such we use
 * the DI version of each which have those encapsulated.
 */
export function runner(filters, container) {
    if (!filters || !filters.length) {
        logger.warn('The client filter runner requires an array of filters to run, received:', filters);
        return true;
    }
    return filters.every((cf) => {
        const target = getFilterByName(cf.name, container);
        return target ? target.filter(cf.parameters) : true;
    });
}
/**
 * Default ClientFilterRunner DI implementation.
 * @public
 */
export class DefaultClientFilterRunner {
    run(filters, container = this.container) {
        return runner(filters, container);
    }
}
__decorate([
    Container
], DefaultClientFilterRunner.prototype, "container", void 0);
/**
 * ClientFilterRunner DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const ClientFilterRunner = DI.createInterface((x) => x.transient(DefaultClientFilterRunner));
