import { DI } from '@microsoft/fast-foundation';
import { logger } from './logger';
/**
 * TimeWindow client filter function.
 * @remarks
 * This will return true between a certain time window. Start and end can be provided in any format recognized by the
 * Date.parse() method.
 *
 * @param start - The start of the time window.
 * @param end - The end of the time window.
 * @public
 */
export function timeWindowFilter(start, end) {
    if (!start && !end) {
        logger.warn('TimeWindow filter parameters must include start, end, or both.');
        return true;
    }
    const now = new Date();
    let result = true;
    if (start) {
        const startDate = new Date(start);
        result = now >= startDate;
    }
    if (end && result) {
        const endDate = new Date(end);
        result = now < endDate;
    }
    return result;
}
/**
 * Default TimeWindow DI implementation.
 * @public
 */
export class DefaultTimeWindow {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params) {
        return timeWindowFilter(params === null || params === void 0 ? void 0 : params.start, params === null || params === void 0 ? void 0 : params.end);
    }
}
/**
 * TimeWindow DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const TimeWindow = DI.createInterface((x) => x.transient(DefaultTimeWindow));
