import { DI } from '@microsoft/fast-foundation';
import UAParser from 'ua-parser-js';
import { logger } from './logger';
/**
 * UserAgent client filter function.
 *
 * @remarks
 * See {@link https://github.com/faisalman/ua-parser-js} for more information.
 *
 * @param browsers - Browser names to target.
 * @param devices - Device types to target.
 * @param oss - Operating system names to target.
 * @param ua - User-agent string (required in nodejs environment).
 * @public
 */
export function userAgentFilter(browsers, devices, oss, ua) {
    if (!browsers && !devices && !oss) {
        logger.warn('UserAgent filter parameters must include browsers, devices, oss, or all three.');
        return true;
    }
    const parser = new UAParser(ua);
    const browser = browsers && parser.getBrowser();
    if (browser && browser.name && browsers.includes(browser.name)) {
        return true;
    }
    const device = devices && parser.getDevice();
    if (device && device.type && devices.includes(device.type)) {
        return true;
    }
    const os = oss && parser.getOS();
    return !!os && !!os.name && oss.includes(os.name);
}
/**
 * Default UserAgent DI implementation.
 * @public
 */
export class DefaultUserAgent {
    /** {@inheritDoc ClientFilter.filter} */
    filter(params) {
        return userAgentFilter(params === null || params === void 0 ? void 0 : params.browsers, params === null || params === void 0 ? void 0 : params.devices, params === null || params === void 0 ? void 0 : params.oss, params === null || params === void 0 ? void 0 : params.ua);
    }
    /** {@inheritDoc UserAgent.isSafari} */
    get isSafari() {
        /**
         * You can use pre-defined params to encapsulate common requests.
         */
        return userAgentFilter(['Safari']);
    }
}
/**
 * UserAgent DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const UserAgent = DI.createInterface((x) => x.transient(DefaultUserAgent));
