import { __decorate, __param } from "tslib";
import { Auth } from '@genesislcap/foundation-comms';
import { DI } from '@microsoft/fast-foundation';
import { logger } from './logger';
/**
 * UserTargeting client filter function.
 * @remarks
 * This will return true when the current user matches on ANY of the parameters. If you wish to match on multiple
 * parameters, consider chaining an array of userTargeting filters together.
 * @param user - A user object.
 * @param usernames - Usernames to target.
 * @param profiles - Profiles to target.
 * @param permissions - Permissions to target.
 * @public
 * @privateRemarks
 * We could provide another parameter to indicate a complete match is required?
 */
export function userTargetingFilter(user, usernames, profiles, permissions) {
    if (!user) {
        logger.warn('UserTargeting filter needs a user to filter on.');
        return true;
    }
    if (!usernames && !profiles && !permissions) {
        logger.warn('UserTargeting filter parameters must include usernames, profiles, permissions, or all three.');
        return true;
    }
    if (usernames && usernames.includes(user.username)) {
        return true;
    }
    if (profiles && profiles.some((p) => user.profiles.includes(p))) {
        return true;
    }
    return !!permissions && permissions.some((p) => user.permissions.includes(p));
}
/**
 * Default UserTargeting DI implementation.
 * @public
 */
let DefaultUserTargeting = class DefaultUserTargeting {
    constructor(auth) {
        this.auth = auth;
    }
    /** {@inheritDoc ClientFilter.filter} */
    filter(params) {
        return userTargetingFilter(this.auth.currentUser, params === null || params === void 0 ? void 0 : params.usernames, params === null || params === void 0 ? void 0 : params.profiles, params === null || params === void 0 ? void 0 : params.permissions);
    }
    /** {@inheritDoc UserTargeting.hasAdminProfile} */
    get hasAdminProfile() {
        /**
         * This could simply be `return this.auth.currentUser.hasAdminProfile()`, but the purpose was to show how we
         * might leverage the baseline filter function with pre-defined params to encapsulate common requests.
         */
        return userTargetingFilter(this.auth.currentUser, undefined, ['ADMIN']);
    }
    /** {@inheritDoc UserTargeting.isLoggedIn} */
    get isLoggedIn() {
        return this.auth.isLoggedIn;
    }
};
DefaultUserTargeting = __decorate([
    __param(0, Auth)
], DefaultUserTargeting);
export { DefaultUserTargeting };
/**
 * UserTargeting DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const UserTargeting = DI.createInterface((x) => x.transient(DefaultUserTargeting));
